package app.main;

import java.util.ArrayList;

import ryulib.OnNotifyEventListener;
import ryulib.game.GameControl;
import ryulib.game.GameControlGroup;
import ryulib.game.GamePlatformInfo;
import ryulib.game.HitArea;
import android.graphics.Paint;
import android.graphics.Point;
import android.view.MotionEvent;

public class Piece extends GameControl {

	public Piece(GameControlGroup gameControlGroup) {
		super(gameControlGroup);
		
		gameControlGroup.addControl(this);
	}
	
	private HitArea _HitArea = new HitArea();
	
	@Override
	protected HitArea getHitArea() {
		return _HitArea;
	}

	private int _X = 0;
	private int _Y = 0;
	private int _MinLeft = 0xFFFF;
	private int _MinTop = 0xFFFF;
	private int _MaxLeft = -1;
	private int _MaxTop = -1;

	ArrayList<Block> _Blocks = new ArrayList<Block>();
	
	public int getWidth() {
		return _MaxLeft - _MinLeft + 1;
	}
		
	public int getHeight() {
		return _MaxTop - _MinTop + 1;
	}
		
	public void clearBlocks() {
		_MinLeft = 0xFFFF;
		_MinTop = 0xFFFF;
		
		_Blocks.clear();
	}

	public void addBlock(int x, int y) {
		if (x < _MinLeft) _MinLeft = x;
		if (y < _MinTop ) _MinTop  = y;
		
		if (x > _MaxLeft) _MaxLeft = x;
		if (y > _MaxTop ) _MaxTop  = y;
		
		Block block = new Block(x, y);
		_Blocks.add(block);
	}

	public void arrange() {
		for (Block block : _Blocks) {
			block.decX(_MinLeft);
			block.decY(_MinTop);
		}

		afterMoved();
	}

	private int _A = 255;
	private int _R = (int) (Math.random() * 100) + 155;
	private int _G = (int) (Math.random() * 100) + 155;
	private int _B = (int) (Math.random() * 100) + 155;
	
	@Override
	protected void onDraw(GamePlatformInfo platformInfo) {
		Paint paint  = platformInfo.getPaint();
		
		if (_isMoving) {
			paint.setARGB(100, _R, _G, _B);
		} else {
			if ((checkCollision(this) != null)) {
				paint.setARGB(100, _R, _G, _B);
			} else {
				paint.setARGB(_A, _R, _G, _B);
			}
		}

		int left = _X*Global.blockSize + _TouchMove.x-_TouchDown.x;
		int top  = _Y*Global.blockSize + _TouchMove.y-_TouchDown.y;
			
		for (Block block : _Blocks) {
			platformInfo.getCanvas().drawRect(
					block.getBoundary().getRect(left, top), 
					paint
			);
		}
	}
	
	// Action Down, Move  , event ߻ ġ
	private Point _TouchDown = new Point();
	private Point _TouchMove = new Point();
	
    private void doActionDown(int x, int y) {
    	_TouchDown.set(x, y);
    	_TouchMove.set(x, y);
    }
    
    private void doActionUp(int x, int y) {
    	int cx = (_X*Global.blockSize) + (x - _TouchDown.x) + (Global.blockSize / 2);
    	int cy = (_Y*Global.blockSize) + (y - _TouchDown.y) + (Global.blockSize / 2); 

    	_TouchDown.set(0, 0);
    	_TouchMove.set(0, 0);

    	cx = (cx / Global.blockSize);
    	cy = (cy / Global.blockSize);
    	
    	if (cx < 0) cx = 0;
    	if (cy < 0) cy = 0;
    	
    	if (cx > (Global.screenWidth  - getWidth()))  cx = Global.screenWidth  - getWidth(); 
    	if (cy > (Global.screenHeight - getHeight())) cy = Global.screenHeight - getHeight(); 

    	setPoint(cx, cy);
    }
    
	private void doActionMove(int x, int y) {
		_TouchMove.set(x, y);
    }
    
	private boolean _isMoving = false;
	
    @Override
    protected boolean onTouchEvent(GamePlatformInfo platformInfo, MotionEvent event) {
    	boolean result = false;
    	
    	int x = (int) event.getX();
    	int y = (int) event.getY();
    	
    	switch (event.getAction()) {
	    	case MotionEvent.ACTION_DOWN: 
	    		if (getIsMyArea(x, y))	{
		    		doActionDown(x, y);
	    	    	_isMoving = true;
		    		result = true;
	    		    bringToFront();
	    		}
	    		break;
	    		
	    	case MotionEvent.ACTION_UP: 
	    		if (_isMoving) {
		    		doActionUp(x, y); 
	    			_isMoving = false;
		    		result = true;
	    		}
	    		break;
	    		
	    	default: 
	    		if (_isMoving) {
	    			doActionMove(x, y); 
		    		result = true;
	    		}
	    		break;
    	}
    	
    	return result;
    }

	private boolean getIsMyArea(int x, int y) {
		for (Block block : _Blocks) {
			if (block.getBoundary(_X, _Y).isMyArea(x, y)) return true;
		}
		return false;
	}
	
	private OnNotifyEventListener _OnMoved = null;
	
	public OnNotifyEventListener getOnMoved() {
		return _OnMoved;
	}

	public void setOnMoved(OnNotifyEventListener _OnMoved) {
		this._OnMoved = _OnMoved;
	}

	private void afterMoved() {
		_HitArea.clear();
		for (Block block : _Blocks) {			
			_HitArea.add(block.getBoundary(_X, _Y));
		}
		
		if (_OnMoved != null) _OnMoved.onNotify(this);
	}
	
	public void setPoint(int x, int y) {		
		_X = x;
		_Y = y;
		afterMoved();
	}

	public void setX(int _X) {
		this._X = _X;
		afterMoved();
	}

	public int getX() {
		return _X;
	}

	public void setY(int _Y) {
		this._Y = _Y;
		afterMoved();
	}

	public int getY() {
		return _Y;
	}
    
}
