package app.main;

import java.util.ArrayList;

import ryulib.OnNotifyEventListener;
import ryulib.game.GameControl;
import ryulib.game.GameControlGroup;
import ryulib.graphic.Boundary;
import android.graphics.Point;

public class PieceFactory extends GameControl {
	
	public PieceFactory(GameControlGroup gameControlGroup) {
		super(gameControlGroup);

	}

	private Boundary[][] _Boundaries = null;
	private int _BoundriesCount;
	
	public void slice() {
		createCells();
		makePieces();
	}

	private void makePieces() {
		_BoundriesCount = Global.boardSize * Global.boardSize;
		while (_BoundriesCount > 0) {
			makePiece();
		}
	}
	
	private void makePiece() {
		int pieceSize  = getRandomPieceSize();
		Piece piece = new Piece(getGameControlGroup());
		Point point = getRandomPointChoice();
		
		addBoundaryToPiece(point, piece);
		pieceSize--;
		_BoundriesCount--;

		while ((pieceSize > 0) && (_BoundriesCount > 0)) {
			point = getNextRandomPoint(point);
			if (point == null) break;
			
			addBoundaryToPiece(point, piece);
			pieceSize--;
			_BoundriesCount--;
		}
		
		piece.arrange();
		
		// ϰ ġϸ鼭  ġ ʵ 100   Ѵ.
		int x, y;
		for (int i=0; i<100; i++) {
			x = Global.getRandom((Global.screenWidth  / Global.blockSize) - piece.getWidth()); 
			y = Global.getRandom((Global.screenHeight / Global.blockSize) - piece.getHeight()); 

			piece.setPoint(x, y);
			if (checkCollision(piece) == null) break;
		}
		
		if (_OnNewPiece != null) _OnNewPiece.onNotify(piece);
	}

	private void createCells() {
		_Boundaries = new Boundary[Global.boardSize][Global.boardSize];
		
		for (int y=0; y<Global.boardSize; y++) {
			for (int x=0; x<Global.boardSize; x++) {
				_Boundaries[x][y] = new Boundary(x*Global.blockSize, y*Global.blockSize, (x+1)*Global.blockSize, (y+1)*Global.blockSize);
			}
		}
	}
	
	private void addBoundaryToPiece(Point point, Piece piece) {
		_Boundaries[point.x][point.y] = null;
		piece.addBlock(point.x, point.y);
	}
	
	private Point getRandomPointChoice() {
		int x = Global.getRandom(Global.boardSize);
		int y = Global.getRandom(Global.boardSize);
		
		while (_Boundaries[x][y] == null) {
			x = Global.getRandom(Global.boardSize);
			y = Global.getRandom(Global.boardSize);
		}
		
		return new Point(x, y);
	}
	
	// ش ǥ Boundary    ִ ?   ǥ  
	private Point getPoint(int x, int y) {
		if ((x < 0) || (x >= Global.boardSize)) return null;
		if ((y < 0) || (y >= Global.boardSize)) return null;
		
		if (_Boundaries[x][y] != null) {
			return new Point(x, y);
		} else {
			return null;
		}
	}

	private Point getNextRandomPoint(Point basePoint) {
		ArrayList<Point> points = new ArrayList<Point>();
		
		Point point;
		
		point = getPoint(basePoint.x-1, basePoint.y);
		if (point != null) points.add(point); 
		
		point = getPoint(basePoint.x+1, basePoint.y);
		if (point != null) points.add(point); 
		
		point = getPoint(basePoint.x, basePoint.y-1);
		if (point != null) points.add(point); 
		
		point = getPoint(basePoint.x, basePoint.y+1);
		if (point != null) points.add(point); 
		
		if (points.size() == 0) {
			return null;
		} else {
			int index = Global.getRandom(points.size());
			return points.get(index);
		}
	}

	private int getRandomPieceSize() {
		// TODO Auto-generated method stub
		return 5;
	}

	public void setOnNewPiece(OnNotifyEventListener _OnNewPiece) {
		this._OnNewPiece = _OnNewPiece;
	}

	public OnNotifyEventListener getOnNewPiece() {
		return _OnNewPiece;
	}

	private OnNotifyEventListener _OnNewPiece = null;

}
