package app.main;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Bitmap.Config;
import ryulib.OnNotifyEventListener;
import ryulib.game.GameControl;
import ryulib.game.GameControlGroup;
import ryulib.game.GamePlatformInfo;

public class TetrisBoard extends GameControl {

	public TetrisBoard(GameControlGroup gameControlGroup) {
		super(gameControlGroup);
		
		gameControlGroup.addControl(this);
		
        _PieceFactory = new PieceFactory(gameControlGroup);
        _PieceFactory.setOnNewPiece(_OnNewPiece);
        
        _PieceList.setOnGameEnd(_OnGameEnd);
	}
	
	private PieceFactory _PieceFactory = null;
	private PieceList _PieceList = new PieceList();

	private OnNotifyEventListener _OnNewPiece = new OnNotifyEventListener() {
		@Override
		public void onNotify(Object sender) {
			getGameControlGroup().addControl((Piece) sender);
			_PieceList.add((Piece) sender);
		}
	};
	
	private OnNotifyEventListener _OnGameEnd = new OnNotifyEventListener() {
		@Override
		public void onNotify(Object sender) {
			System.exit(0);
		}
	};
	
	private Bitmap _BoardBitmap = Bitmap.createBitmap(1, 1, Config.ARGB_8888);
	private Canvas _BoardCanvas = new Canvas();

	private Canvas _Canvas = null;
	private Paint _Paint = new Paint();
	
	public void startGame() {
		_PieceList.clear();
		_PieceFactory.slice();
	}
	
	@Override
	protected void onStart(GamePlatformInfo platformInfo) {
		_Canvas = platformInfo.getCanvas();

        Global.setScreenSize(_Canvas.getWidth(), _Canvas.getHeight());
		setBoardSize(Global.blockSize * Global.boardSize);
		
		startGame();
	}
	
	@Override
	protected void onDraw(GamePlatformInfo platformInfo) {
		_Paint.setARGB(255, 0, 0, 0);
		_Canvas.drawRect(platformInfo.getRect(), _Paint);
		
		_Canvas.drawBitmap(_BoardBitmap, 0, 0, _Paint);		
	}
	
	private int _BoardSize = 0;

	public int getBoardSize() {
		return _BoardSize;
	}

	public void setBoardSize(int _BoardSize) {
		this._BoardSize = _BoardSize;
		doPrepareBoardBitmap(_BoardSize);		
	}

	private void doPrepareBoardBitmap(int boardSize) {
		_BoardBitmap = Bitmap.createBitmap(boardSize+1, boardSize+1, Config.ARGB_8888);
		_BoardCanvas.setBitmap(_BoardBitmap);		
		
		_Paint.setARGB(255, 55, 55, 55);

		for (int x=0; x<=Global.boardSize; x++) {
			_BoardCanvas.drawLine(x*Global.blockSize, 0, x*Global.blockSize, boardSize, _Paint);
		}
		
		for (int y=0; y<=Global.boardSize; y++) {
			_BoardCanvas.drawLine(0, y*Global.blockSize, boardSize, y*Global.blockSize, _Paint);
		}
	}

}
